#version 420 core
precision highp float;
layout (location = 0) in vec3 i_position;
layout (location = 1) in vec3 i_normal;
layout (location = 2) in vec2 i_texcoord;
layout (location = 3) in float i_texture_id;
layout (location = 4) in float i_lightmap;

layout (binding = 2) uniform samplerCubeArray point_light_shadow_maps;

struct PointLight
{
	vec4 colour_brightness;
	vec4 position_cubemapindex;
};

layout (std140, binding = 0) uniform Matrices
{
	mat4 view_matrix;
	mat4 proj_matrix;
	mat4 model_matrix;
	mat4 mvp_matrix;
	vec4 colour_;
	vec4 tex_atlas_mul_xy;
	vec4 fog_colour_distance;
	ivec4 receivesshadows_unlit;
	ivec4 n_lights_by_type;
	PointLight point_lights[64];
};
out vec3 world_position;
out vec3 normal;
out vec3 colour;
out vec2 texcoord;
out float texture_id;
out float lightmap;
out float fog_contribution;

vec3 light_dir = normalize(vec3(3, 0, -10));


void main()
{
	world_position = vec3(model_matrix * vec4(i_position, 1.0f));
	vec3 normal_transformed = normalize(i_normal * mat3(model_matrix));

	
	float brightness = dot(-light_dir, normal_transformed);
	brightness = brightness * float(brightness > 0);
	brightness *= 0.7;
	brightness += 0.5;

	colour = vec3(1, 1, 1) * brightness * float(brightness > 0) * vec3(colour_);
	colour = min(colour, vec3(colour_));
	texcoord = i_texcoord;
	normal = normal_transformed;
	texture_id = i_texture_id;
	lightmap = i_lightmap;
	
	fog_contribution = length(view_matrix * vec4(world_position, 1)) / fog_colour_distance.w;
	fog_contribution = clamp(fog_contribution, 0.0f, 1.0f);

	colour = vec3(1, 1, 1);
	colour *= brightness * float(brightness > 0);
	colour *= vec3(colour_);
	//colour = mix(colour, vec3(fog_colour_distance), fog_contribution_a);
}