#version 420 core
precision mediump float;

uniform vec4 line_colour;
out vec4 frag_color;
in vec4 f_actual_position;
in vec3 f_world_position;
in vec3 f_normal;
in vec3 f_colour;
noperspective in vec2 f_texcoord;
in vec2  f_texcoord_perspective;
in float f_texture_id;
in float f_lightmap;
in float f_fog_contribution;

layout (binding = 0) uniform sampler2D colour_texture;
layout (binding = 1) uniform sampler2D dithering_lut;
layout (binding = 2) uniform samplerCubeArray point_light_shadow_maps;

struct PointLight
{
	vec4 colour_brightness;
	vec4 position_cubemapindex;
};

layout (std140, binding = 0) uniform Matrices
{
	mat4 view_matrix;
	mat4 proj_matrix;
	mat4 model_matrix;
	mat4 mvp_matrix;
	vec4 colour_;
	vec4 tex_atlas_mul_xy;
	vec4 fog_colour_distance;
	ivec4 receivesshadows_unlit;
	ivec4 n_lights_by_type;
	PointLight point_lights[64];
};

void main()
{	
	//Calculate point light value
	vec3 point_light_contribution = vec3(0.0f);
	for (int i = 0; i < n_lights_by_type.x; i++)
	{
		vec3 light_to_surface = f_world_position - point_lights[i].position_cubemapindex.xyz;
		float light_world_distance = length(light_to_surface);

		if (light_world_distance < 300.f)
		{
			float normal_impact = clamp(dot(-f_normal, light_to_surface), 0.0f, 1.0f);
			bool in_shadow = false;
			//Calculate shadow
			if (point_lights[i].position_cubemapindex.w >= 0.0f && receivesshadows_unlit.x == 1)
			{
				//Get distance from light to fragment position

				//Sample depth map
				float depth_map_distance = texture(point_light_shadow_maps, vec4(light_to_surface.xyz, point_lights[i].position_cubemapindex.w)).r;
				depth_map_distance *= 300.0f;
				float c1 = light_world_distance / 300.f;
				float c2 = depth_map_distance / 300.f;

				//Determine whether in shadow or not
				in_shadow = (depth_map_distance < light_world_distance - (4.0f * normal_impact * normal_impact));
			}

			//Calculate light brightness
			if (in_shadow == false)
			{
				float distance_squared = dot(light_to_surface, light_to_surface);
				float attenuation = 1.0f / (distance_squared);
				float falloff = clamp((1.0f - (light_world_distance / 300.f)) * 4, 0.0f, 1.0f);
				float brightness = point_lights[i].colour_brightness.w * attenuation * normal_impact * falloff;
				
				point_light_contribution += point_lights[i].colour_brightness.xyz * brightness;
			}
		}
	}

	float warping_strength = 0.15f;
	vec2 sample_texcoord = mix(f_texcoord_perspective, f_texcoord, warping_strength);

	if (isinf(sample_texcoord.x) || isinf(sample_texcoord.y))
	{
		discard;
		sample_texcoord = vec2(0.0f);
	}

	sample_texcoord = mod(sample_texcoord, vec2(1.0f));
//	if (tex_atlas_mul_xy.w > 0.0f)
//	{
//		sample_texcoord.x += mod(f_texture_id, tex_atlas_mul_xy.z);
//		sample_texcoord.y += floor(f_texture_id / tex_atlas_mul_xy.z);
//		sample_texcoord *= vec2(tex_atlas_mul_xy);
//	}
	vec4 texture_sample = texture(colour_texture, sample_texcoord);
	if (texture_sample.a * colour_.a < texture(dithering_lut, gl_FragCoord.xy / 8).r) // alpha clipping
	{
		discard;
	}
	float lightmap_corrected = 255 - f_lightmap / 255.0;

	frag_color = texture_sample * (receivesshadows_unlit.y == 1 ? vec4(1,1,1,1) : vec4(f_colour + point_light_contribution, 1));// * vec4(lightmap_corrected, lightmap_corrected, lightmap_corrected, 1);
	frag_color = mix(frag_color, vec4(fog_colour_distance.xyz, 1), f_fog_contribution);
}