#version 420 core
precision mediump float;

uniform vec4 line_colour;
out vec4 frag_color;
in vec3 world_position;
in vec3 colour;
in vec2 texcoord;
in float lightmap;

uniform sampler2D tex;

struct PointLight
{
	vec4 colour_brightness;
	vec4 position;
};

layout (std140, binding = 0) uniform Matrices
{
	mat4 view_matrix;
	mat4 proj_matrix;
	mat4 model_matrix;
	mat4 mvp_matrix;
	vec4 colour_;
	vec4 tex_atlas_mul_xy;
	vec4 fog_colour_distance;
	ivec4 receivesshadows_unlit;
	ivec4 n_lights_by_type;
	PointLight point_lights[128];
};

void main()
{
	vec3 point_light_contribution = vec3(0.0f);
	for (int i = 0; i < n_lights_by_type.x; i++)
	{
		vec3 light_to_pos = world_position - point_lights[i].position.xyz;
		float distance_squared = dot(light_to_pos, light_to_pos);
		float attenuation = 1 / (distance_squared);
		float brightness = point_lights[i].colour_brightness.w * attenuation;
		
		point_light_contribution += point_lights[i].colour_brightness.xyz * brightness;
	}

	vec4 texture_sample = texture(tex, texcoord);
	if (texture_sample.a < 0.5f) // alpha clipping
	{
		discard;
	}
	float lightmap_corrected = 255 - lightmap / 255.0;
	frag_color = texture_sample * vec4(colour + point_light_contribution, 1);// * vec4(lightmap_corrected, lightmap_corrected, lightmap_corrected, 1);
	//frag_color = vec4(point_light_contribution, 1);
}