#version 420 core
precision mediump float;
layout (location = 0) in vec3 i_position;
layout (location = 1) in vec3 i_normal;
layout (location = 2) in vec2 i_texcoord;
layout (location = 3) in float i_lightmap;

uniform int frames;
uniform int current_frame;
uniform float uv_correction_padding_x;

struct PointLight
{
	vec4 colour_brightness;
	vec4 position;
};

layout (std140, binding = 0) uniform Matrices
{
	mat4 view_matrix;
	mat4 proj_matrix;
	mat4 model_matrix;
	mat4 mvp_matrix;
	vec4 colour_;
	vec4 tex_atlas_mul_xy;
	vec4 fog_colour_distance;
	ivec4 receivesshadows_unlit;
	ivec4 n_lights_by_type;
	PointLight point_lights[128];
};
out vec3 world_position;
out vec3 colour;
out vec2 texcoord;
out float lightmap;

vec3 light_dir = normalize(vec3(3, 0, -10));

void main()
{
	world_position = vec3(model_matrix * vec4(i_position, 1.0f));
	gl_Position = proj_matrix * view_matrix * model_matrix * vec4(i_position, 1.0f);
	vec3 normal_transformed = normalize(i_normal * mat3(model_matrix));

	
	float brightness = dot(-light_dir, normal_transformed);
	brightness = brightness * float(brightness > 0);
	brightness *= 0.7;
	brightness += 0.5;
	//colour = vec3(1, 1, 1) * brightness * float(brightness > 0) * vec3(colour_);
    colour = vec3(colour_);
	texcoord.x = (float(current_frame) / float(frames)) + (i_texcoord.x / frames);
	texcoord.x *= uv_correction_padding_x;
    texcoord.y = i_texcoord.y;

	lightmap = i_lightmap;
}